Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
  .getService(Components.interfaces.mozIJSSubScriptLoader)
  .loadSubScript("chrome://global/content/nsUserSettings.js");


function GoalStar() {
    this.registerObserver();
}

GoalStar.prototype = {
  //strings: null,
  
  /* observer handling */
  get _os() {
    return Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  },
  registerObserver: function() {
    this._os.addObserver(this, "manager-status", false);
  },
  unregisterObserver: function() {
    this._os.removeObserver(this, "manager-status");
  },
  
  onLoad: function() {
    // initialization code
    this.initialized = true;
    
    var ver = -1, firstrun = true;
    var current = '';
    
    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]  
                            .getService(Components.interfaces.nsIXULAppInfo);  
    var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]  
                                   .getService(Components.interfaces.nsIVersionComparator);  
    
    if(versionChecker.compare(appInfo.version, "3.9") > 0) {  
         // running under Firefox 4.0 or later  
        Components.utils.import("resource://gre/modules/AddonManager.jsm");  
        AddonManager.getAddonByID("goalstar@pixcept.de", function(addon) {  
          current = addon.version;
        });
    } else {
        var gExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
                                .getService(Components.interfaces.nsIExtensionManager);
        current = gExtensionManager.getItemForID("goalstar@pixcept.de").version;
    }
    
    var Prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    Prefs = Prefs.getBranch("extensions.goalstar@pixcept.de.");
      
    try {
      ver = Prefs.getCharPref("version");
      firstrun = Prefs.getBoolPref("firstrun");
    } catch(e) {
    } finally{
      if (firstrun){
        Prefs.setBoolPref("firstrun",false);
        Prefs.setCharPref("version",current);
        window.openDialog("chrome://goalstar/content/options.xul","OptionDialog", "chrome")
        /*window.setTimeout(function(){
          gBrowser.selectedTab = gBrowser.addTab("http://www.fmscene.de/firstrun?version=" + current);
        }, 1500);*/
      }   
      if (ver!=current && !firstrun){
        Prefs.setCharPref("version",current);
    }
    }
    window.removeEventListener("load",function(){ Overlay.init(); },true);
    
    var service = Components.classes["@pixcept.de/goalstar/update-data;1"].getService().wrappedJSObject;
    service.getData();
  },
  
  onUnload: function() {
    goalstar.unregisterObserver();
  },

  onMenuItemCommand: function(e) {
  },

  onToolbarButtonCommand: function(e) {
    // just reuse the function above.  you can change this, obviously!
    goalstar.onMenuItemCommand(e);
  },
  
  onStatusClick: function(e) {
    
    if (e.button == 2) {
          window.openDialog("chrome://goalstar/content/options.xul","OptionDialog", "chrome")
          return;
    } else if(e.button == 0) {
        var service = Components.classes["@pixcept.de/goalstar/update-data;1"].getService().wrappedJSObject;
        //service.storeLogin();
        var userName = nsPreferences.copyUnicharPref("extensions.goalstar.username", "");
        if(!userName || userName == '') {
          window.openDialog("chrome://goalstar/content/options.xul","OptionDialog", "chrome")
          return;
        }
        var password = service.getLogin(userName);
        if(!password || password == '') {
          window.openDialog("chrome://goalstar/content/options.xul","OptionDialog", "chrome")
        }
        
        var browser = getBrowser();
        var content = 'target=action%3Dlive&loginname=' + encodeURIComponent(userName) + '&passwrd=' + encodeURIComponent(password);
        var referrer = null;  
        
        var stringStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
                           .createInstance(Components.interfaces.nsIStringInputStream);
        if ("data" in stringStream) // Gecko 1.9 or newer
            stringStream.data = content;
        else // 1.8 or older
            stringStream.setData(content, content.length);

        var sessID = service.genMD5(content);
        var postData = Components.classes["@mozilla.org/network/mime-input-stream;1"]
                       .createInstance(Components.interfaces.nsIMIMEInputStream);
        
        postData.addHeader("Content-Type", "application/x-www-form-urlencoded");
        postData.addContentLength = true;
        postData.setData(stringStream);
                       
        var flags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;  
        
        var newTab = browser.addTab();
        browser.selectedTab = newTab;
        
        var game_host = service.getStatusData('gamehost');
        if(!game_host) game_host = 'www.goalstar.de';
        
        loadURI('http://' + game_host + '/?action=login&do=send&smID=' + sessID, referrer, postData, false);
    }
    return;
  },
  
  observe: function(statusData, topic, data) {
    if(topic != "manager-status")
      return;
    
    var service = Components.classes["@pixcept.de/goalstar/update-data;1"].getService().wrappedJSObject;
    
    statusData = statusData.wrappedJSObject;
    
    if(data == "updated") {
        // component updated the status data, we can now process it
        var txtObj = document.getElementById('goalstar-statustext');
        if(!txtObj) return;
        
        var teamname = service.getStatusData('teamname');
        if(!teamname) return;
        
        var data = new Array();
        var statusText = "";
        
        var els = ['team', 'manager', 'userid', 'gameid', 'server', 'minute', 'versus', 'goals', 'vsgoals', 'goals45', 'vsgoals45', 'athome'];
        for(var i = 0; i < els.length; i++) {
          var key = els[i];
          data[key] = service.getStatusData(key);
        }
        
        if(!data['gameid']) {
            statusText = "&smstatus.nolivegame;";
            statusText = this._replaceLocales(statusText);
            statusText = statusText.replace("{TEAM}", teamname);
        } else {
            if(data['athome'] == 1) {
                statusText = teamname + " " + data['goals'] + " : " + data['vsgoals'] + " " + data['versus'];
            } else {
                statusText = data['versus'] + " " + data['goals'] + " : " + data['vsgoals'] + " " + teamname;
            }
            statusText += " (" + data['minute'] + ". &smstatus.matchminute;)";
            statusText = this._replaceLocales(statusText);
        }
        
        txtObj.setAttribute('value', statusText);
    }
  },

  _replaceLocales: function(str) {
    var re = /&([0-9a-z_\.-]+)(;|\b)/gi;
    while(ma = re.exec(str)) {
      var repl = this._getLocalizedString(ma[1]);
      str = str.replace(ma[0], repl);
    }

    return str;
  },

  _getLocalizedString: function(str) {
    var ret = "";
    try {
      var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
                  .getService(Components.interfaces.nsIStringBundleService);
      var stringBundle = sbs.createBundle("chrome://goalstar/locale/overlay.properties");
      ret = stringBundle.GetStringFromName(str);
    } catch(e) {}
    return ret || str;
  }

};

var goalstar = new GoalStar();

window.addEventListener("load", goalstar.onLoad, false);
window.addEventListener("unload", goalstar.onUnload, false);
