
// import the pref helpers
Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
  .getService(Components.interfaces.mozIJSSubScriptLoader)
  .loadSubScript("chrome://global/content/nsUserSettings.js");


function FetchData() {
  pxFWTools.LOG('FetchData() constructor');
  this.wrappedJSObject = this;
  this.items = [];
}
FetchData.prototype = {
  classID: Components.ID("{ea7e2380-09e9-11e0-81e0-0800200c9a66}"),
  contractID: "@pixcept.de/goalstar/status-data;1",
  classDescription: "Fetch status data and process it.",

  wrappedJSObject: null,
  
  sServiceUrl: "http://www.goalstar.de/service.php",
  sPostData: "",
  htmlData: "",
  channel: null,
  mCallbackFunc: null,
  
  
  init: function(callback) {
    pxFWTools.LOG('FetchData init');
    
    this.mCallbackFunc = callback;
    var userName = nsPreferences.copyUnicharPref("extensions.goalstar.username", "");
    if(!userName || userName == "") return;
    
    var userPass = dataUpdater.getLogin(userName);
    if(!userPass || userPass == "") return;
    
    var userHash = dataUpdater.genHash(userName, userPass);
    
    this.sPostData = 'manager=' + encodeURIComponent(userName) + '&hash=' + encodeURIComponent(userHash);
    pxFWTools.LOG('Sending data ' + this.sPostData);
    
    //this.notify("data-loading");
    var iosvc = Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Components.interfaces.nsIIOService);
    
    var uri = iosvc.newURI(this.sServiceUrl, null, null);  
    this.channel = iosvc.newChannelFromURI(uri);
    
    var inputStream = Components.classes["@mozilla.org/io/string-input-stream;1"]  
                      .createInstance(Components.interfaces.nsIStringInputStream);  
    inputStream.setData(this.sPostData, this.sPostData.length);  
    
    var httpChannel = this.channel.QueryInterface(Components.interfaces.nsIHttpChannel);  
    //httpChannel.referrer = referrerURI;  
    
    var uploadChannel = httpChannel.QueryInterface(Components.interfaces.nsIUploadChannel);  
    uploadChannel.setUploadStream(inputStream, "application/x-www-form-urlencoded", -1);  
   
    // order important - setUploadStream resets to PUT  
    uploadChannel.requestMethod = "POST";  
    
    uploadChannel.asyncOpen(this, null);
  },

  // nsIStreamListener
  onDataAvailable: function(request, context, inputStream, offset, aLength) {
	try {
      var scriptableInputStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
          .createInstance(Components.interfaces.nsIScriptableInputStream);  
      scriptableInputStream.init(inputStream);  
      
      this.htmlData += scriptableInputStream.read(aLength); 
      pxFWTools.LOG('onDataAvailable: ' + offset + ' :: ' + aLength);
	} catch(e) { pxFWTools.LOG('onDataAvailable failed with error ' + e); }
  },

  // nsIRequestObserver
  onStartRequest: function(request, context) {
  try {
      this.htmlData = "";
      pxFWTools.LOG('onStartRequest run');
  } catch(e) { pxFWTools.LOG('onStartRequest failed with error ' + e); }

  },
  onStopRequest: function(request, context, aStatus) {
	try {
      if(Components.isSuccessCode(aStatus)) {  
        // request was successfull  
        this.mCallbackFunc(this.htmlData);  
      } else {  
        // request failed  
        this.mCallbackFunc(null);  
      }
      this.channel = null;
      pxFWTools.LOG('onSopRequest: ' + aStatus);
      this.notify('updated');
	} catch(e) {pxFWTools.LOG('onStopRequest failed with error ' + e); }
  },

  // nsIChannelEventSink  
  onChannelRedirect: function(aOldChannel, aNewChannel, aFlags) {  
    // if redirecting, store the new channel  
    this.channel = aNewChannel;  
  }, 

  // nsIInterfaceRequestor  
  getInterface: function (aIID) {  
    try {  
      return this.QueryInterface(aIID);  
    } catch (e) {  
      throw Components.results.NS_NOINTERFACE;  
    }  
  },

  // nsIProgressEventSink (not implementing will cause annoying exceptions)  
  onProgress : function (aRequest, aContext, aProgress, aProgressMax) { },  
  onStatus : function (aRequest, aContext, aStatus, aStatusArg) { },  
  
  // nsIHttpEventSink (not implementing will cause annoying exceptions)  
  onRedirect : function (aOldChannel, aNewChannel) { },  
  
  notify: function(message) {
    var os = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    os.notifyObservers(this, "manager-status", message);
  },

  QueryInterface : function(aIID) {  
    if (aIID.equals(Components.interfaces.nsISupports) ||  
        aIID.equals(Components.interfaces.nsIInterfaceRequestor) ||  
        aIID.equals(Components.interfaces.nsIChannelEventSink) ||   
        aIID.equals(Components.interfaces.nsIProgressEventSink) ||  
        aIID.equals(Components.interfaces.nsIHttpEventSink) ||  
        aIID.equals(Components.interfaces.nsIStreamListener))  
      return this;  
  
    throw Components.results.NS_NOINTERFACE;  
  }
};

function PXFWTools() {
  this.setInterval = function(func, delay) {
    var timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
    var callback = { that: this, notify: function () {
           func.call(this.that);
        }
    };
    pxFWTools.LOG('setInterval called with timeout ' + delay);
    timer.initWithCallback(callback, delay, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
    return timer;
  };
  
  this.LOG = function(msg) {
    if(!nsPreferences.getBoolPref("extensions.goalstar.log", false))
      return;
    msg = "goalstar:\t" + new Date() + "\n" + msg;
    var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                           .getService(Components.interfaces.nsIConsoleService);
    consoleService.logStringMessage(msg);
    return msg;
  };
}

var pxFWTools = new PXFWTools();
pxFWTools.globObj = this;

function DataUpdater() {
  this.statusData = [];
  this.wrappedJSObject = this;
  this._registerObserver();
  this.initUpdateCheck();
}
DataUpdater.prototype = {
  wrappedJSObject: null,
  statusData: null,
  _branch: null,
  
  getStatusData: function(key) {
      var ret = null;
      if(this.statusData[key]) ret = this.statusData[key];
      
      pxFWTools.LOG('statusData with key ' + key + ' requested: ' + ret);
      return ret;
  },
  
  storeData: function(sData) {
      pxFWTools.LOG('FetchData sent us some html data: ' + sData);
      
      //dataUpdater.initUpdateCheck();
      
      this.statusData = [];
      
      var aLines = sData.split(/\n/);
      if(aLines.length < 1) return;
      if(aLines[0] != "OK") {
          pxFWTools.LOG('ERROR: ' + aLines[0]);
          return;
      }
      for(var i = 1; i < aLines.length; i++) {
          var aTmp = aLines[i].split(':');
          var key = aTmp.shift();
          var value = aTmp.join(':');
          
          this.statusData[key] = value;
          
          pxFWTools.LOG('KEY: ' + key + ', VALUE: ' + value);
      }
  },
  
  getData: function() {
      pxFWTools.LOG('getData() called');
      var statusdata = new FetchData();
      pxFWTools.LOG('FetchData Object created');

      pxFWTools.LOG('doing init now');
      statusdata.init(function(sData) { dataUpdater.storeData(sData); });
    
      pxFWTools.LOG('doing initUpdateCheck now');
  },
  
  storeLogin: function() {
      var userName = nsPreferences.copyUnicharPref("extensions.goalstar.username", "");
      var userPass = nsPreferences.copyUnicharPref("extensions.goalstar.password", "");
      
      if(!userName || userName == "" || !userPass || userPass == "")
          return;
      
      this.removeLogin();
      
      var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",  
                                                   Components.interfaces.nsILoginInfo,  
                                                   "init");  
      var loginInfo = new nsLoginInfo('chrome://goalstar',
                                      'chrome://goalstar', null,
                                      userName, userPass, 'svc_user', 'svc_pw');
      var passwordManager = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager); 
      passwordManager.addLogin(loginInfo);
  },
  
  removeLogin: function() {
      try {
         // Get Login Manager   
         var passwordManager = Components.classes["@mozilla.org/login-manager;1"]
                               .getService(Components.interfaces.nsILoginManager);
         
         // Find users for this extension
         var logins = passwordManager.findLogins({}, 'chrome://goalstar', 'chrome://goalstar', null);  
         
         for (var i = 0; i < logins.length; i++) {  
            passwordManager.removeLogin(logins[i]);  
         }
      }
      catch(ex) {  
         // This will only happen if there is no nsILoginManager component class  
      }
  },
  
  getLogin: function(findUser) {
      try {
         // Get Login Manager   
         var passwordManager = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager);  
         
         // Find users for the given parameters  
         var logins = passwordManager.findLogins({}, 'chrome://goalstar', 'chrome://goalstar', null);  
         
         // Find user from returned array of nsILoginInfo objects  
         for (var i = 0; i < logins.length; i++) {  
            if (logins[i].username == findUser) {  
               return logins[i].password;
            }  
         }  
      }  
      catch(ex) {  
         // This will only happen if there is no nsILoginManager component class  
      }
      
      return '';
  },
  
  genMD5: function(dataString) {
      var converter =  
        Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].  
          createInstance(Components.interfaces.nsIScriptableUnicodeConverter);  
        
      converter.charset = "ISO-8859-1";  
      var result = {};  
      
      var data = converter.convertToByteArray(dataString, result);  
      var ch = Components.classes["@mozilla.org/security/hash;1"]  
                         .createInstance(Components.interfaces.nsICryptoHash);  
      ch.init(ch.MD5);  
      ch.update(data, data.length);  
      
      var hash = ch.finish(false);  
        
      // return the two-digit hexadecimal code for a byte  
      function toHexString(charCode)  
      {  
        return ("0" + charCode.toString(16)).slice(-2);  
      }  
        
      // convert the binary hash data to a hex string.  
      return [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");  
  },
  
  genHash: function(userName, password) {
      var dataString = userName.toLowerCase() + '::' + this.genMD5(password) + '::nfyext10';
      return this.genMD5(dataString);
  },
  
  _registerObserver: function() {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]  
                                .getService(Components.interfaces.nsIPrefService);  
  
    this._branch = prefService.getBranch("extensions.goalstar.");  
  
    this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);  
  
    this._branch.addObserver("", this, false);  
  },
  
  unregister: function() {
    if (!this._branch) return;  
    this._branch.removeObserver("", this);
  },
  
  observe: function(aSubject, aTopic, aData) {  
    if(aTopic != "nsPref:changed") return;  
    // aSubject is the nsIPrefBranch we're observing (after appropriate QI)  
    // aData is the name of the pref that's been changed (relative to aSubject)  
    if(aData == "password") {
      //pxFWTools.LOG('Password was changed in preferences');
      
      this.storeLogin();
      
      //nsPreferences.setUnicharPref('extensions.goalstar.password', '');
    }
  },
  
  initUpdateCheck: function() {
    // get the interval in seconds, with a minimum of 1 minute
    pxFWTools.LOG('doing initUpdateCheck now');
    pxFWTools.timer = pxFWTools.setInterval(function() {
      pxFWTools.LOG('getData timer starts now.');
      dataUpdater.getData();
    }, 30 * 1000);
  },

  QueryInterface: function(aIID) {
    if(aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  }
};
var dataUpdater = new DataUpdater(); // make sure we're a singleton even when not used as a service
function __DataUpdater() {
  return dataUpdater;
};
__DataUpdater.prototype = {
  classID: Components.ID("{fa5a0b70-09e9-11e0-81e0-0800200c9a66}"),
  contractID: "@pixcept.de/goalstar/update-data;1",
  classDescription: "Fetch status data"
};


if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([FetchData, __DataUpdater]);
else
    var NSGetModule = XPCOMUtils.generateNSGetModule([FetchData, __DataUpdater]);







/*

# var browser = document.getElementById('browser');  
   1. var content = encodeURIComponent('password=foobar');  
   2. var referrer = null;  
   3. var postData = Components.classes['@mozilla.org/io/string-input-stream;1']  
   4.                .createInstance(Components.interfaces.nsIStringInputStream);  
   5. content = 'Content-Type: application/x-www-form-urlencoded\n'+  
   6.           'Content-Length: '+content.length+'\n\n'+  
   7.           content;  
   8. postData.setData(content, content.length);  
   9. var flags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;  
  10. browser.loadURIWithFlags('http://piro.sakura.ne.jp/', flags, referrer, null, postData);  
  
  */
